// Generated by CoffeeScript 1.4.0
(function() {
  var escodegen, nodeIsInfinity, nodeIsNumericLiteral, nodeIsUnaryMinus, numberProperty, replaceAllNodes, tools, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  escodegen = require('escodegen');

  tools = require('./tools');

  _ = require('underscore');

  numberProperty = function(property) {
    return {
      type: 'MemberExpression',
      computed: false,
      object: {
        type: 'Identifier',
        name: 'Number'
      },
      property: {
        type: 'Identifier',
        name: property
      }
    };
  };

  replaceAllNodes = function(_arg) {
    var ast, predicate, replacement;
    ast = _arg.ast, predicate = _arg.predicate, replacement = _arg.replacement;
    return exports.traverse(ast, function(node) {
      if (predicate(node)) {
        return tools.replaceProperties(node, replacement(node));
      }
    });
  };

  nodeIsInfinity = function(node) {
    return node.type === 'Literal' && node.value === Infinity;
  };

  nodeIsNumericLiteral = function(node) {
    return node.type === 'Literal' && typeof node.value === 'number';
  };

  nodeIsUnaryMinus = function(node) {
    return node.type === 'UnaryExpression' && node.operator === '-';
  };

  exports.createLiteral = function(value) {
    if (typeof value === 'number' && value < 0) {
      return {
        type: 'UnaryExpression',
        operator: '-',
        argument: {
          type: 'Literal',
          value: -value
        }
      };
    } else {
      return {
        type: 'Literal',
        value: value
      };
    }
  };

  exports.isNumericLiteral = function(node) {
    return nodeIsNumericLiteral(node) || (nodeIsUnaryMinus(node) && nodeIsNumericLiteral(node.argument));
  };

  exports.evalLiteral = function(node) {
    if (nodeIsNumericLiteral(node)) {
      return node.value;
    } else if (nodeIsUnaryMinus(node) && nodeIsNumericLiteral(node.argument)) {
      return -node.argument.value;
    } else {
      throw "not a numeric literal";
    }
  };

  exports.evalLiterals = function(ast, evals) {
    var format, _results;
    format = true;
    _results = [];
    while (format) {
      format = false;
      _results.push(exports.traverse(ast, function(node) {
        return evals.forEach(function(n) {
          if (n.test(node)) {
            exports.replaceWithLiteral(node, n["eval"](node));
            return format = true;
          }
        });
      }));
    }
    return _results;
  };

  exports.replaceWithLiteral = function(node, value) {
    return tools.replaceProperties(node, exports.createLiteral(value));
  };

  exports.replaceNegativeInfinities = function(ast) {
    return replaceAllNodes({
      ast: ast,
      predicate: function(node) {
        return nodeIsUnaryMinus(node) && nodeIsInfinity(node.argument);
      },
      replacement: function() {
        return numberProperty('NEGATIVE_INFINITY');
      }
    });
  };

  exports.replacePositiveInfinities = function(ast) {
    return replaceAllNodes({
      ast: ast,
      predicate: nodeIsInfinity,
      replacement: function() {
        return numberProperty('POSITIVE_INFINITY');
      }
    });
  };

  exports.coverageNode = function(node, filename, identifier) {
    return {
      type: 'ExpressionStatement',
      expression: {
        type: 'UpdateExpression',
        operator: '++',
        prefix: false,
        argument: {
          type: 'MemberExpression',
          computed: true,
          property: exports.createLiteral(node.loc.start.line),
          object: {
            type: 'MemberExpression',
            computed: true,
            object: {
              type: 'Identifier',
              name: identifier
            },
            property: exports.createLiteral(filename)
          }
        }
      }
    };
  };

  exports.traverse = function(ast, filter, callback) {
    if (!(callback != null)) {
      callback = filter;
      filter = [];
    }
    return escodegen.traverse(ast, {
      enter: function(node) {
        var _ref;
        if (filter.length === 0 || (_ref = node.type, __indexOf.call(filter, _ref) >= 0)) {
          return callback(node);
        }
      }
    });
  };

  exports.addBeforeEveryStatement = function(ast, addback) {
    exports.traverse(ast, function(node) {
      var _ref;
      if (node.type === 'IfStatement') {
        ['consequent', 'alternate'].forEach(function(path) {
          if ((node[path] != null) && node[path].type !== 'BlockStatement') {
            return node[path] = {
              type: 'BlockStatement',
              body: [node[path]]
            };
          }
        });
      }
      if (((_ref = node.type) === 'ForInStatement' || _ref === 'ForStatement' || _ref === 'WhileStatement' || _ref === 'WithStatement' || _ref === 'DoWhileStatement') && (node.body != null) && node.body.type !== 'BlockStatement') {
        return node.body = {
          type: 'BlockStatement',
          body: [node.body]
        };
      }
    });
    return exports.traverse(ast, function(node) {
      var _ref;
      if ((_ref = node.type) === 'BlockStatement' || _ref === 'Program') {
        node.body = _.flatten(node.body.map(function(statement) {
          var _ref1, _ref2;
          if (((_ref1 = statement.expression) != null ? _ref1.type : void 0) === 'FunctionExpression') {
            return [addback(statement.expression), statement];
          } else if (((_ref2 = statement.expression) != null ? _ref2.type : void 0) === 'CallExpression') {
            return [addback(statement.expression), statement];
          } else if (statement.type === 'FunctionDeclaration') {
            return [addback(statement.body), statement];
          } else {
            return [addback(statement), statement];
          }
        }));
      }
      if (node.type === 'SwitchCase') {
        return node.consequent = _.flatten(node.consequent.map(function(statement) {
          return [addback(statement), statement];
        }));
      }
    });
  };

}).call(this);
